<?php

/**
 * Carerix Wordpress Plugin
 *
 * @category settings
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @since Sep 3, 2013 4:27:07 PM
 * @version $Rev$
 */

/**
 * Campaign class
 * Used to ....
 */
class Frontend_Shortcode_Campaign {

    static $count = 1;
    private $params = array();
    private $template_name = 'cxWpForm';
    private $required_fields = array(
        'campaign', 'name', 'email'
    );

    public function __construct() {
        
    }

    public function run($params) {
        $backtrace = debug_backtrace();
        foreach ($backtrace as $bkt) {
            if (isset($bkt['file'])) {
                if (basename($bkt['file']) == 'class-frontend.php')
                    return '';
            }
        }
        $this->params = $params;
        if (isset($params['sendto'])) {
            $receiverEmails = explode('|', trim($params['sendto']));
        } else {
            echo '<span style="color: red;">The sendto parameter is required</span>';
            return;
        }


        $errors = array();
        $general_error = '';
        $input_data = filter_input_array(INPUT_POST, FILTER_SANITIZE_STRING);

        $container = 'c_' . Frontend_Shortcode_Campaign::$count++;
        if (isset($input_data[$container]))
            $input_data = $input_data[$container];
        else
            $input_data = NULL;
        if ($input_data !== NULL && wp_verify_nonce($input_data['_carerix_nonce_field'], 'campaign')) {
            foreach ($this->required_fields as $field) {
                if (!isset($input_data[$field]) && strlen(trim($input_data[$field]) == 0)) {
                    $errors[$field] = 'This field is required';
                }
            }

            if (count($errors) == 0) {
                foreach ($receiverEmails as $receiverEmail) {
                    $campaignID = $input_data['campaign'];
                    $senderName = $input_data['name'];
                    $senderEmail = $input_data['email'];
                    $this->applyToCampaign($campaignID, $senderName, $senderEmail, $receiverEmail, $input_data);
                }
            }
        }

        if (isset($params['codeinname']) && trim($params['codeinname']) !== '') {
            $filterValue = explode('|', $params['codeinname']);
            $filterType = 'name';
        } elseif (isset($params['campaignid']) && trim($params['campaignid']) !== '') {
            $filterValue = explode('|', $params['campaignid']);
            $filterType = 'id';
        } else {
            $filterValue = false;
            $filterType = 'all';
        }

        $data = array();
        $data['container'] = $container;
        $data['campaigns'] = $this->getCampaignsForSelect($filterType, $filterValue);
        $data['salutations'] = $this->get_salutations();
        $data['functionLevels'] = $this->getFunctionLevels();
        $data['functionLevelsJson'] = json_encode($this->getFunctionLevels());
        $data['disabled'] = '';
        if (count($data['campaigns']) == 0)
            $data['disabled'] = 'disabled';

        ob_start();


        $template = new Utils_Template('frontend/forms/campaign');
        $template->with($data)->display();

        $out = ob_get_contents();
        ob_clean();
        return $out;
    }

    private function getCampaignsForSelect($filterType, $filterValue) {
        $campaigns = array();

        $camps = $this->getCampaigns($filterType, $filterValue);
        foreach ($camps as $camp) {
            $startDate = date("Y-m-d", strtotime($camp->startDate));
            $campaigns[] = array('value' => $camp->todoID, 'label' => $camp->name . ' / ' . $startDate);
        }

        return $campaigns;
    }

    private function getCampaigns($filterType, $filterValue) {

        $deadline = date("Y-m-d H:i:s O", strtotime("today"));

        $qualifier = "deleted=0 AND todoTypeKey=22 
            AND toStatusNode.parentNodes.value='CampaignActiveTag' 
            AND (deadline == nil OR (deadline > (NSCalendarDate)'$deadline'))";

        $params = array('show' => array('', 'toStatusNode.value'), 'qualifier' => $qualifier, 'start' => 0, 'count' => 999);

        $general_exception = false;
        try {
            $campaigns = Carerix_Api_Rest_Entity_CRToDo::findAll($params);
        } catch (Carerix_Api_Rest_Exception $care) {
            $general_exception = $care;
        } catch (Exception $ex) {
            $general_exception = $ex;
        }
        if ($general_exception) {
            Utils_Log::write(Utils_Log::TYPE_DATANODE, $campaigns, $general_exception);
        }

        if ($filterType == 'all') {
            return $campaigns;
        }

        $keep = array();
        if ($campaigns !== null) {
            switch ($filterType) {
                case 'name':
                    foreach ($campaigns as $campaign) {
                        foreach ($filterValue as $code) {
                            if (stripos($campaign->toStatusNode->value, html_entity_decode($code)) !== FALSE) {
                                array_push($keep, $campaign);
                            }
                        }
                    }
                    break;
                case 'id':
                    foreach ($campaigns as $campaign) {
                        if (in_array($campaign->todoID, $filterValue)) {
                            array_push($keep, $campaign);
                        }
                    }
                    break;
            }
        }

        return $keep;
    }

    private function get_salutations() {
        $processedGenders = array();

        $general_exception = false;
        try {
            $genders = Carerix_Api_Rest_Entity_CRDataNode::listBy(array('type' => 'Salutation'));
            foreach ($genders as $gender) {
                $processedGenders[] = array('value' => $gender->dataNodeID, 'label' => $gender->value);
            }
        } catch (Carerix_Api_Rest_Exception $care) {
            $general_exception = $care;
        } catch (Exception $ex) {
            $general_exception = $ex;
        }
        if ($general_exception) {
            Utils_Log::write(Utils_Log::TYPE_DATANODE, $genders, $general_exception);
        }

        return $processedGenders;
    }

    private function getFunctionLevels() {
	    $FUNCTION_LEVEL_CACHE = wp_cache_get( 'function_level_cache', CARERIX_CACHE_GROUP );
	    if ( false !== $FUNCTION_LEVEL_CACHE ) {
	    	return $FUNCTION_LEVEL_CACHE;
	    }

        try {
            $fieldsLevel0 = Carerix_Api_Rest_Entity_CRDataNode::listBy(array('type' => 'Business line'));
        } catch (Carerix_Api_Rest_Exception $care) {
            Utils_Log::write(Utils_Log::TYPE_DATANODE, $fieldsLevel0, $care);
        }

        $fieldsHierarchy = array();

        foreach ($fieldsLevel0 as $field) {
            $level = array('id' => $field->dataNodeID, 'value' => $field->value, 'children' => array());
            $fieldsLevel1 = null;
            try {
                $fieldsLevel1 = Carerix_Api_Rest_Entity_CRDataNode::listBy(array('type' => 'Function0', 'parent' => $field->dataNodeID));
            } catch (Carerix_Api_Rest_Exception $care) {
                Utils_Log::write(Utils_Log::TYPE_DATANODE, $fieldsLevel1, $care);
            }


            if ($fieldsLevel1 != null) {
                foreach ($fieldsLevel1 as $fieldLevel1) {
                    $level2 = array('id' => $fieldLevel1->dataNodeID, 'value' => $fieldLevel1->value, 'children' => array());
                    $fieldsLevel2 = null;
                    try {
                        $fieldsLevel2 = Carerix_Api_Rest_Entity_CRDataNode::listBy(array('type' => 'Function1', 'parent' => $fieldLevel1->dataNodeID));
                    } catch (Carerix_Api_Rest_Exception $care) {
                        Utils_Log::write(Utils_Log::TYPE_DATANODE, $fieldsLevel2, $care);
                    }

                    if ($fieldsLevel2 !== null) {
                        foreach ($fieldsLevel2 as $fieldLevel2) {
                            $level3 = array('id' => $fieldLevel2->dataNodeID, 'value' => $fieldLevel2->value);
                            array_push($level2['children'], $level3);
                        }
                    }
                    array_push($level['children'], $level2);
                }
            }

            array_push($fieldsHierarchy, $level);
        }

	    wp_cache_set( 'function_level_cache', $fieldsHierarchy, CARERIX_CACHE_GROUP, 600 ); // server side caching for 60 secs

        return $fieldsHierarchy;
    }

    private function applyToCampaign($campaignID, $senderName, $senderEmail, $receiverEmail, $data) {
        $cxtransformed = array();
        foreach ($data as $key => $value) {
            $cxtransformed['cx_' . $key] = $value;
        }

        if (!$senderEmail)
            $senderEmail = get_bloginfo('admin_email');
        if (!$senderName)
            $senderName = get_bloginfo('name');
        $parameters = array(
            'from' => $senderEmail,
            'to' => $receiverEmail,
            'emailtemplate' => $this->template_name,
            'cx_campaign_name' => '',
            'name' => $senderName,
            'ID' => (int) $campaignID,
        );
        $parameters = array_merge($parameters, $cxtransformed);

        try {
            //$campaign = new Carerix_Api_Rest_Entity_CRCampaign();

            $campaign = Carerix_Api_Rest_Entity_CRCampaign::find($campaignID, array('show' => array('name', 'subject')));

            $parameters['cx_campaign_name'] = $campaign->getSubject();
//            $campaign = new Carerix_Api_Rest_Entity_CRCampaign();
            $campaign->setId((int) $campaignID);
            $campaign->email($parameters);
        } catch (Exception $exception) {
            Utils_Log::write('CampaignSubscribe', @$campaign, $exception);
            $this->normalEmail($senderEmail, $senderName, $receiverEmail, $data);
        }
    }

    private function normalEmail($senderEmail, $senderName, $receiverEmail, $data) {

        $subject = $data['salutation_name'] . ' ' . @$data['name'] . ' subscribed to campaign ' . @$data['campaign_name'] . ' / ' . @$data['campaign_start_date'];

        $mailContent = '<table>';
        $mailContent .= '<tr><td>Name: </td><td>' . htmlspecialchars(@$data['name']) . '</td></tr>';
        $mailContent .= '<tr><td>Function: </td><td>' . htmlspecialchars(@$data['function']) . '</td></tr>';
        $mailContent .= '<tr><td>Organization: </td><td>' . htmlspecialchars(@$data['organization']) . '</td></tr>';
        $mailContent .= '<tr><td>Email: </td><td>' . htmlspecialchars(@$data['email']) . '</td></tr>';
        $mailContent .= '<tr><td>Phone: </td><td>' . htmlspecialchars(@$data['phone']) . '</td></tr>';
        $mailContent .= '<tr><td>Message: </td><td>' . htmlspecialchars(@$data['cmessage']) . '</td></tr>';
        $mailContent .= '</table>';

        $headers = 'MIME-Version: 1.0' . "\r\n";
        $headers .= 'Content-type: text/html; charset=iso-8859-1' . "\r\n";
        $headers .= 'From: ' . htmlspecialchars(@$data['name']) . ' <' . htmlspecialchars(@$data['email']) . '>' . "\r\n";


        $sendMail = mail(
                $receiverEmail, $subject, $mailContent, $headers
        );

        return $sendMail;
    }

}
