<?php /** @noinspection SqlResolve */

class Utils_CronCleaner {
    /**
     * @var WP_Interfacer;
     */
    private $interfacer  = null;
    
    private $source_id = 0;
    private $retention_buffer_days = 3;
	private $manual_called = false;

	public function __construct($source_id, $manual_called = false) {
        $this->interfacer = WP_Interfacer::getInstance();
        $this->source_id = $source_id;
        $this->manual_called = $manual_called;
    }
    
    public function clean() {
        $postMetaTable = $this->interfacer->db_get_postmeta_name();
        $query = "DELETE FROM `$postMetaTable` WHERE `meta_key` = 'source_cron' and `meta_value` = {$this->source_id}";
        $this->interfacer->db_query($query);
    }

	/**
	 * Finds all posts (e.g. jobs) that are not in the retrieved wp_feed anymore and remove them
	 *
	 * @param bool $type
	 *
	 * @return bool|void
	 */
    public function run($type = false) {
        if($type === false || !$this->source_id) return;

        $postMetaTable = $this->interfacer->db_get_postmeta_name();

        if($this->interfacer->wpml_is_active()) {
            // multilanguage
            $languages = $this->interfacer->get_wpml_active_languages();
	        $languages = array_keys($languages);

	        foreach($languages as $language) {
                $language = trim(strtolower($language));

	            if ( Environment_Settings::getRestApiSync() == "on" ) {
		            $jobSource = new Entity_Source_Job($this->source_id);
		            $sourceData = $jobSource->getData();
		            $sourceData['settings'] = unserialize($sourceData['settings']);
		            $jobCron = new Entity_Cron_Job($sourceData);
		            $jobCron->setPage(0);
		            $jobCron->setRegisterLanguage($language);
		            $synced_date = $jobCron->getCxIsSyncedToThisDateTime($this->source_id);
		            $arrPubIDs = $jobCron->retrieveInactivePublicationIdsByRestApi($synced_date, $this->retention_buffer_days);
		            if ($this->manual_called) {
		            	$medium_code = $jobCron->getMediumCode();
			            echo "\n<p>";
			            echo "\nPROCESSING EXPIRED/DELETED for mediumcode `{$medium_code}`<br/>";
			            echo "\nJobs detected as expired/deleted in the last {$this->retention_buffer_days} days: " . count($arrPubIDs) . "<br/>";
			            echo "\n</p>";
			            echo "\n";
		            }
		            $arrPubIDs[] = -1; // To enforce a correct query
		            $lstPubIDs = implode(",", $arrPubIDs);
		            $query = $this->sqlStatementPostsToDelete($postMetaTable, $this->source_id, $type, "guid", $lstPubIDs, $language);
	            } else {
		            $query = $this->sqlStatementPostsToDelete($postMetaTable, $this->source_id, $type, "source", $this->source_id, $language);
		            $checkBroughtPublicationsSql = "
		                SELECT post_id FROM `{$postMetaTable}`
		                WHERE `meta_key` = 'source_cron'
		                AND `meta_value` = {$this->source_id}";
		            $currentBatch = $this->interfacer->db_get_custom($checkBroughtPublicationsSql);
		            if(count($currentBatch) <= 0 ) return false;

//	            $query = "
//                SELECT * FROM `$postMetaTable`
//                    WHERE `meta_key` = 'source'
//                    AND `meta_value` = {$this->source_id}
//                    AND `post_id` IN
//                (SELECT post_id
//                      FROM `$postMetaTable`
//                      WHERE
//                        `meta_key` LIKE '%_carerix_{$type}_{$language}'
//                      AND `post_id` NOT IN
//                        (SELECT post_id FROM `$postMetaTable`
//                            WHERE
//                                `meta_key`   = 'source_cron'
//                                AND
//                                `meta_value` = {$this->source_id}))";
	            }


                $results = $this->interfacer->db_get_custom($query);
                foreach($results as $result) {
                    $this->interfacer->entity_delete($result['post_id']);
                    do_action('cxwordpress_post_deleted', $result['post_id']);
                } 
            }
        } else {
			// single language
        	if ( Environment_Settings::getRestApiSync() == "on" ) {
		        $jobSource = new Entity_Source_Job($this->source_id);
		        $sourceData = $jobSource->getData();
		        $sourceData['settings'] = unserialize($sourceData['settings']);

		        $jobCron = new Entity_Cron_Job($sourceData);
		        $jobCron->setPage(0); // unused in this case
		        $synced_date = $jobCron->getCxIsSyncedToThisDateTime($this->source_id);
		        $arrPubIDs = $jobCron->retrieveInactivePublicationIdsByRestApi($synced_date, $this->retention_buffer_days); // "2017-01-01 00:00:00"

		        if ($this->manual_called) {
			        $medium_code = $jobCron->getMediumCode();
			        echo "\n<p>";
			        echo "\nPROCESSING EXPIRED/DELETED for mediumcode `{$medium_code}`<br/>";
			        echo "\nJobs detected as expired/deleted in the last {$this->retention_buffer_days} days: " . count($arrPubIDs) . "<br/>";
			        echo "\n</p>";
			        echo "\n";
		        }

		        $arrPubIDs[] = -1; // To enforce a correct query
		        $lstPubIDs = implode(",", $arrPubIDs);

		        $query = $this->sqlStatementPostsToDelete($postMetaTable, $this->source_id, $type, "guid", $lstPubIDs);


//		        $query = "
//	            SELECT * FROM `{$postMetaTable}`
//	                WHERE `meta_key` = 'guid'
//	                AND `meta_value` IN ($lstPubIDs)
//	                AND `post_id` IN
//	            (SELECT post_id
//	                  FROM `$postMetaTable`
//	                  WHERE
//	                    `meta_key` LIKE '%_carerix_{$type}'
//	                  AND `post_id` NOT IN
//	                    (SELECT post_id FROM `{$postMetaTable}`
//	                        WHERE
//	                            `meta_key`   = 'source_cron'
//	                            AND
//	                            `meta_value` = {$this->source_id}))";
	        } else {
        		$query = $this->sqlStatementPostsToDelete($postMetaTable, $this->source_id, $type, "source", $this->source_id);
//		        $query = "
//	            SELECT * FROM `{$postMetaTable}`
//	                WHERE `meta_key` = 'source'
//	                AND `meta_value` IN ({$this->source_id})
//	                AND `post_id` IN
//	            (SELECT post_id
//	                  FROM `$postMetaTable`
//	                  WHERE
//	                    `meta_key` LIKE '%_carerix_{$type}'
//	                  AND `post_id` NOT IN
//	                    (SELECT post_id FROM `{$postMetaTable}`
//	                        WHERE
//	                            `meta_key`   = 'source_cron'
//	                            AND
//	                            `meta_value` = {$this->source_id}))";
		        $checkBroughtPublicationsSql = "
                SELECT post_id FROM `{$postMetaTable}`
                WHERE `meta_key` = 'source_cron'
                AND `meta_value` = {$this->source_id}";

		        if ($type !== 'candidate') {
			        $currentBatch = $this->interfacer->db_get_custom($checkBroughtPublicationsSql);
			        if(count($currentBatch) <= 0 ) return false;
		        }
	        }

            $results = $this->interfacer->db_get_custom($query);
            foreach($results as $result) {
	            $this->interfacer->entity_delete($result['post_id'], true);
                do_action('cxwordpress_post_deleted', $result['post_id']);
            }
        }
    }


	public function setCxIsSyncedToThisDateTime($source_id)
	{
		if ( Environment_Settings::getRestApiSync() !== "on" ) return;

		$lastSyncDirPath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . 'syncstatus';
		$sync_timestamp_file = $lastSyncDirPath . DIRECTORY_SEPARATOR . "last_sync_jobs_SRCID{$source_id}.txt";

		$nl_datetime = DateTime::createFromFormat(
			'Y-m-d G:i:s',
			current_time("mysql", 1),
			new DateTimeZone('UTC')
		);
		$nl_datetime->setTimeZone(new DateTimeZone('Europe/Amsterdam')); // CX calcs with the NL-timezone internally

		$handle = fopen( $sync_timestamp_file, "w" );
		fwrite( $handle, $nl_datetime->format("Y-m-d G:i:00") );
		fclose( $handle );
	}

	public function clearRestApiSyncStatuses()
	{
		$lastSyncDirPath = Carerix::$TMPDIR . DIRECTORY_SEPARATOR . 'syncstatus';

		if (!file_exists($lastSyncDirPath)) return;

		$files = glob($lastSyncDirPath . '/*.txt'); // get all file names
		foreach ($files as $file) { // iterate files
			if (is_file($file)) unlink($file); // delete file
		}
	}

	/*
	 * Build query for deleting the (job)posts which aren't available anymore (expired/deleted)
	 *
	 */
	/**
	 * @since 2.2.0
	 *
	 * @param string    $postMetaTable
	 * @param int       $source_id
	 * @param string    $type
	 * @param string    $meta_key
	 * @param string    $meta_values      (list of values)
	 * @param string    $language
	 *
	 * @return string
	 */
	private function sqlStatementPostsToDelete($postMetaTable, $source_id, $type, $meta_key, $meta_values, $language = "")
	{
		if ( strlen($language) ) {
			$language_postfix = "_{$language}";
		} else {
			$language_postfix = "";
		}

		if ($meta_key == "guid") {
			$extra_clause = "";
		} else {
			$extra_clause = " AND `post_id` NOT IN 
                    (SELECT post_id FROM `{$postMetaTable}` 
                        WHERE 
                            `meta_key`   = 'source_cron'
                            AND
                            `meta_value` = {$source_id})";
		}

		$query = "
            SELECT * FROM `{$postMetaTable}`
                WHERE `meta_key` = '{$meta_key}'
                AND `meta_value` IN ({$meta_values})
                AND `post_id` IN
            (SELECT post_id 
                  FROM `$postMetaTable` 
                  WHERE 
                    `meta_key` LIKE '%_carerix_{$type}{$language_postfix}'
                	{$extra_clause}
            )";
		return $query;
	}

}