<?php /** @noinspection PhpUndefinedConstantInspection */

namespace Carerix\Cookie;

/**
 * Sets/reads/deletes cookies
 * Supports storing arrays and objects too
 *
 * @author Ruben <ruben@carerix.com>
 */
class Cookie
{
	const EXPIRY_SECONDS = 30*24*3600;


	public static function setCookie($name, $value, $expiry_seconds = self::EXPIRY_SECONDS ) {
		$secure = ( 'https' === parse_url( wp_login_url(), PHP_URL_SCHEME ) );
		$_COOKIE[$name] = json_encode($value); // force cookie to exist in current request (Else it's available the next page request)
		setcookie($name, json_encode($value), time() + $expiry_seconds, COOKIEPATH, COOKIE_DOMAIN, $secure);
		if ( SITECOOKIEPATH !== COOKIEPATH ) {
			setcookie($name, json_encode($value), time() + $expiry_seconds, SITECOOKIEPATH, COOKIE_DOMAIN, $secure);
		}
	}

	public static function getCookie( $name, $default_value = null, $as_associative_array = false) {
		$ret_value = $default_value;
		if (isset($_COOKIE[$name])) $ret_value = $_COOKIE[$name];
		$mixed_val = json_decode(stripslashes($ret_value), $as_associative_array);
		if (isset($mixed_val)) $ret_value = $mixed_val;

		return $ret_value;
	}

	public static function deleteCookie( $name ) {
		$secure = ( 'https' === parse_url( wp_login_url(), PHP_URL_SCHEME ) );
		unset($_COOKIE[$name]);
		setcookie($name, null,  time()-7200, COOKIEPATH, COOKIE_DOMAIN, $secure);
		if ( SITECOOKIEPATH !== COOKIEPATH ) {
			setcookie($name, null,  time()-7200, SITECOOKIEPATH, COOKIE_DOMAIN, $secure);
		}
	}
}